=== added file 'build.sh'
--- build.sh	1970-01-01 00:00:00 +0000
+++ build.sh	2016-01-29 13:54:55 +0000
@@ -0,0 +1,50 @@
+#!/usr/bin/env bash
+
+###
+# http://stackoverflow.com/a/23342259
+###
+exe() { echo "\$ $@" ; "$@" ; }
+
+show_usage () {
+  echo ""
+  echo "Set up the building of libssts and clean up too."
+  echo "  ./build.sh clean  -  Clean up the project and build files."
+  echo ""
+  echo "  ./build.sh build  -  Create the build directory, run CMake (if"
+  echo "                       needed) and build libssts."
+  echo ""
+  echo "  ./build.sh help   -  Show this helpful help text."
+}
+
+do_build () {
+  if ( ! -d "./build" )
+  then
+    exe mkdir ./build
+  fi
+  
+  cd ./build
+  
+  if [ ! -f "Makefile" ]
+  then
+    exe cmake ..
+  fi
+  
+  make
+}
+
+do_clean () {
+  if [ -d "./build" ]
+  then
+    exe rm -R ./build
+  fi
+}
+
+case $1 in
+     "help"|"-h") show_usage ;;
+     "clean") do_clean ;;
+     "build"|"") do_build ;;
+     *) show_usage ;;
+esac
+
+
+

=== modified file 'libssts/CMakeLists.txt'
--- libssts/CMakeLists.txt	2015-09-15 15:18:49 +0000
+++ libssts/CMakeLists.txt	2016-01-28 20:12:20 +0000
@@ -32,6 +32,8 @@
     ${CMAKE_CURRENT_SOURCE_DIR}/Map.h
     ${CMAKE_CURRENT_SOURCE_DIR}/Matrix.c
     ${CMAKE_CURRENT_SOURCE_DIR}/Matrix.h
+    ${CMAKE_CURRENT_SOURCE_DIR}/MainLoop.c
+    ${CMAKE_CURRENT_SOURCE_DIR}/MainLoop.h
     ${CMAKE_CURRENT_SOURCE_DIR}/Thread.c
     ${CMAKE_CURRENT_SOURCE_DIR}/Thread.h
     ${CMAKE_CURRENT_SOURCE_DIR}/SimpleTypeSystem.c

=== modified file 'libssts/Func.h'
--- libssts/Func.h	2015-11-14 17:26:42 +0000
+++ libssts/Func.h	2016-01-28 20:12:20 +0000
@@ -52,6 +52,7 @@
 #define CALLBACK(k) ((Callback) k)
 #define FOREACHFUNC(k) ((ForEachFunc) k)
 #define COMPFUNC(k) ((CompFunc) k)
+#define RUNFUNC(k) ((RunFunc) k)
 
 
 /*

=== modified file 'libssts/LinkedList.c'
--- libssts/LinkedList.c	2016-01-26 13:21:36 +0000
+++ libssts/LinkedList.c	2016-01-29 17:49:29 +0000
@@ -168,7 +168,6 @@
 sboolean
 s_linked_list_next (SLinkedList * self) {
   if (self->current == NULL) {
-    s_err_print ("Provided SLinkedList is NULL.");
     return FALSE;
   }
   if (self->current->next != NULL) {
@@ -184,7 +183,6 @@
 sboolean
 s_linked_list_prev (SLinkedList * self) {
   if (self->current == NULL) {
-    s_err_print ("Provided SLinkedList is NULL.");
     return FALSE;
   }
   if (self->current->prev != NULL) {

=== modified file 'libssts/MainLoop.c'
--- libssts/MainLoop.c	2013-09-04 20:52:28 +0000
+++ libssts/MainLoop.c	2016-01-29 17:49:29 +0000
@@ -0,0 +1,461 @@
+#include "MainLoop.h"
+#include "Thread.h"
+#include "LinkedList.h"
+
+/*
+Copyright (c) 2013-2016 Gustav Hartvigsson
+
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+ */
+
+/*
+ * The default mainloop, it is hidden.
+ */
+SMainLoop *
+_default_main_loop = NULL;
+
+/*
+ * The items that are held by the linked lists in SMainLoop.
+ */
+typedef struct SMainLoopItem {
+  SMainLoopState state; /* in what list should this item be? */
+  sboolean is_alive; /* If this is TRUE then this can be removed. */
+
+  RunFunc callback; /* The callback that is run. */
+  spointer user_data; /* Pointer to the user data that was provided. */
+} SMainLoopItem;
+
+typedef struct  SMainLoopTeardownHandler {
+  Callback teardown_handler;
+
+  spointer obj_self;
+  spointer user_data;
+
+} SMainLoopTeardownHandler;
+
+/* This is the freeing function for MainLoopTeardownHandler objcets it also
+ * calls the teardown handler.
+ *
+ * This reduces the iteration time for the teardown.
+ */
+void
+_s_main_loop_teardown_handerer_obj_free (SMainLoopTeardownHandler * self);
+
+struct SMainLoop {
+  suint ref_count;
+  SMainLoopState ring_1_state;
+  SMainLoopState ring_2_state;
+  sboolean is_run;
+
+  SMutex * mutex[S_MAIN_LOOP_RING_LAST][S_MAIN_LOOP_STATE_LAST];
+
+  SLinkedList * ring_1_event_handler_list;
+  SLinkedList * ring_1_io_push_list;
+  SLinkedList * ring_1_io_pull_list;
+  SLinkedList * ring_1_workers_list;
+
+  SLinkedList * ring_2_event_handler_list;
+  SLinkedList * ring_2_io_push_list;
+  SLinkedList * ring_2_io_pull_list;
+  SLinkedList * ring_2_workers_list;
+  
+  SLinkedList * teardown_handler_list;
+  
+  SMutex * teardown_handler_mutex;
+};
+
+schar *
+s_main_loop_state_get_name (SMainLoopState state) {
+  return SMainLoopStateName[state];
+}
+
+schar *
+s_main_loop_ring_get_name (SMainLoopRing ring) {
+  return SMainLoopStateName[ring];
+}
+
+/* ************************************************************************** */
+
+void
+_s_main_loop_do_list (SLinkedList * event_handler_list);
+
+void
+_s_main_loop_do_ring_two (SMainLoop * self);
+
+/* -----------------------------------------------------------------------------
+ * imprementation of the real functions and stuffs.
+ * -----------------------------------------------------------------------------
+ */
+
+SMainLoop *
+s_main_loop_new () {
+  SMainLoop * self = malloc (sizeof (SMainLoop));
+
+  for (sint i = 0; i < S_MAIN_LOOP_RING_LAST; i++){
+    for (sint j = 0; j < S_MAIN_LOOP_STATE_LAST; j++){
+      self->mutex[i][j] = s_mutex_new ();
+    }
+  }
+  
+  self->teardown_handler_mutex = s_mutex_new ();
+  
+  s_mutex_lock (self->mutex[S_MAIN_LOOP_RING_NONE][S_MAIN_LOOP_STATE_NONE]);
+
+  self->ref_count = 1;
+  self->ring_1_state = S_MAIN_LOOP_STATE_EVENT;
+  self->ring_2_state = S_MAIN_LOOP_STATE_EVENT;
+  self->is_run = FALSE;
+  
+  self->ring_1_event_handler_list = s_linked_list_new (FREEFUNC (free));
+  self->ring_1_io_push_list = s_linked_list_new (FREEFUNC (free));
+  self->ring_1_io_pull_list = s_linked_list_new (FREEFUNC (free));
+  self->ring_1_workers_list = s_linked_list_new (FREEFUNC (free));
+
+  self->ring_2_event_handler_list = s_linked_list_new (FREEFUNC (free));
+  self->ring_2_io_push_list = s_linked_list_new (FREEFUNC (free));
+  self->ring_2_io_pull_list = s_linked_list_new (FREEFUNC (free));
+  self->ring_2_workers_list = s_linked_list_new (FREEFUNC (free));
+
+  self->teardown_handler_list = s_linked_list_new (FREEFUNC (_s_main_loop_teardown_handerer_obj_free));
+
+  s_mutex_unlock (self->mutex[S_MAIN_LOOP_RING_NONE][S_MAIN_LOOP_STATE_NONE]);
+
+  return self;
+}
+
+SMainLoop *
+s_main_loop_get_default () {
+  if (_default_main_loop == NULL) {
+    _default_main_loop = s_main_loop_new ();
+  }
+  return _default_main_loop;
+}
+
+void
+s_main_loop_run (SMainLoop * self) {
+  assert (self != NULL);
+  self->is_run = TRUE;
+  while (self->is_run) {
+    switch (self->ring_1_state) {
+      case (S_MAIN_LOOP_STATE_EVENT):
+        _s_main_loop_do_list (self->ring_1_event_handler_list);
+        self->ring_1_state = S_MAIN_LOOP_STATE_IO_PUSH;
+        break;
+      case (S_MAIN_LOOP_STATE_IO_PUSH):
+        _s_main_loop_do_list (self->ring_1_io_push_list);
+        self->ring_1_state = S_MAIN_LOOP_STATE_IO_PULL;
+        break;
+      case (S_MAIN_LOOP_STATE_IO_PULL):
+        _s_main_loop_do_list (self->ring_1_io_pull_list);
+        self->ring_1_state = S_MAIN_LOOP_STATE_RING_TWO_NEXT;
+        break;
+      case (S_MAIN_LOOP_STATE_RING_TWO_NEXT):
+        _s_main_loop_do_ring_two (self);
+        self->ring_1_state = S_MAIN_LOOP_STATE_DO_WORKERS;
+        break;
+      case (S_MAIN_LOOP_STATE_DO_WORKERS):
+        _s_main_loop_do_list (self->ring_1_workers_list);
+        self->ring_1_state = S_MAIN_LOOP_STATE_SLEEP;
+        break;
+      case (S_MAIN_LOOP_STATE_SLEEP):
+        s_usleep (1);
+        self->ring_1_state = S_MAIN_LOOP_STATE_EVENT;
+        break;
+      default:
+        s_err_print ("Reaching a place that should not be reached.\n"
+                     "Aborting.");
+        exit (666);
+    }
+    s_mutex_lock (self->mutex[S_MAIN_LOOP_RING_NONE][S_MAIN_LOOP_STATE_NONE]);
+
+    sint no_of_handlers = 0;
+
+    no_of_handlers += s_linked_list_len (self->ring_1_event_handler_list);
+    no_of_handlers += s_linked_list_len (self->ring_1_io_push_list);
+    no_of_handlers += s_linked_list_len (self->ring_1_io_pull_list);
+    no_of_handlers += s_linked_list_len (self->ring_1_workers_list);
+
+    no_of_handlers += s_linked_list_len (self->ring_2_event_handler_list);
+    no_of_handlers += s_linked_list_len (self->ring_2_io_push_list);
+    no_of_handlers += s_linked_list_len (self->ring_2_io_pull_list);
+    no_of_handlers += s_linked_list_len (self->ring_2_workers_list);
+
+    if (!self->is_run || no_of_handlers == 0) {
+      s_mutex_unlock (self->mutex[S_MAIN_LOOP_RING_NONE][S_MAIN_LOOP_STATE_NONE]);
+      break;
+    }
+    s_mutex_unlock (self->mutex[S_MAIN_LOOP_RING_NONE][S_MAIN_LOOP_STATE_NONE]);
+  }
+}
+
+
+void
+s_main_loop_quit (SMainLoop * self) {
+  s_mutex_lock (self->mutex[S_MAIN_LOOP_RING_NONE][S_MAIN_LOOP_STATE_NONE]);
+  self->is_run = FALSE;
+
+  s_linked_list_free (self->ring_1_event_handler_list, TRUE);
+  s_linked_list_free (self->ring_1_io_push_list, TRUE);
+  s_linked_list_free (self->ring_1_io_pull_list, TRUE);
+  s_linked_list_free (self->ring_1_workers_list, TRUE);
+
+  s_linked_list_free (self->ring_2_event_handler_list, TRUE);
+  s_linked_list_free (self->ring_2_io_push_list, TRUE);
+  s_linked_list_free (self->ring_2_io_pull_list, TRUE);
+  s_linked_list_free (self->ring_2_workers_list, TRUE);
+
+  s_linked_list_free (self->teardown_handler_list, TRUE);
+
+  for (sint i = 0; i < S_MAIN_LOOP_RING_LAST; i++) {
+    for (sint j = 0; j < S_MAIN_LOOP_STATE_LAST; j++) {
+      s_print ("[%d][%d]\n",i,j);
+      s_mutex_free (self->mutex[i][j]);
+    }
+  }
+  
+  s_mutex_free (self->teardown_handler_mutex);
+  
+  free (self);
+}
+
+void
+s_main_loop_ref (SMainLoop * self) {
+  self->ref_count++;
+}
+
+void
+s_main_loop_unref (SMainLoop * self) {
+  self->ref_count--;
+}
+
+void
+s_main_loop_add_event_handler (SMainLoop * self,
+                               SMainLoopRing ring,
+                               RunFunc event_handler,
+                               spointer user_data) {
+  //
+  s_mutex_lock (self->mutex[ring][S_MAIN_LOOP_STATE_EVENT]);
+  if (user_data == NULL) {
+    s_err_print ("No user data provided to the event handeler."
+                 " This is an error.");
+    print_backtrace ();
+    return;
+  }
+
+  SMainLoopItem * item = malloc (sizeof (SMainLoopItem));
+  item->state = S_MAIN_LOOP_STATE_EVENT;
+  item->callback = event_handler;
+  item->user_data = user_data;
+  item->is_alive = TRUE;
+
+  switch (ring) {
+    case (S_MAIN_LOOP_RING_ONE):
+      s_linked_list_append (self->ring_1_event_handler_list, item);
+      break;
+    case (S_MAIN_LOOP_RING_TWO):
+      s_linked_list_append (self->ring_2_event_handler_list, item);
+      break;
+    default:
+      s_err_print ("Invalid value of the ring parameter for "
+                   "s_main_loop_add_io_pull function.");
+  }
+  s_mutex_unlock (self->mutex[ring][S_MAIN_LOOP_STATE_EVENT]);
+}
+
+void
+s_main_loop_add_io_push (SMainLoop * self,
+                         SMainLoopRing ring,
+                         RunFunc push_callback,
+                         spointer user_data) {
+  //
+  s_mutex_lock (self->mutex[ring][S_MAIN_LOOP_STATE_IO_PUSH]);
+  if (user_data == NULL) {
+    s_err_print ("No user data provided to the IO push event."
+                 " This is an error.");
+    print_backtrace ();
+    return;
+  }
+
+  SMainLoopItem * item = malloc (sizeof (SMainLoopItem));
+  item->state = S_MAIN_LOOP_STATE_IO_PUSH;
+  item->callback = push_callback;
+  item->user_data = user_data;
+  item->is_alive = TRUE;
+
+  switch (ring) {
+    case (S_MAIN_LOOP_RING_ONE):
+      s_linked_list_append (self->ring_1_io_push_list, item);
+      break;
+    case (S_MAIN_LOOP_RING_TWO):
+      s_linked_list_append (self->ring_2_io_push_list, item);
+      break;
+    default:
+      s_err_print ("Invalid value of the ring parameter for "
+                   "s_main_loop_add_io_push function.");
+  }
+  s_mutex_unlock (self->mutex[ring][S_MAIN_LOOP_STATE_IO_PUSH]);
+}
+
+void
+s_main_loop_add_io_pull (SMainLoop * self,
+                         SMainLoopRing ring,
+                         RunFunc pull_callback,
+                         spointer user_data) {
+  //
+  s_mutex_lock (self->mutex[ring][S_MAIN_LOOP_STATE_IO_PULL]);
+  if (user_data == NULL) {
+    s_err_print ("No user data provided to the IO pull event."
+                 " This is an error.");
+    print_backtrace ();
+    return;
+  }
+
+  SMainLoopItem * item = malloc (sizeof (SMainLoopItem));
+  item->state = S_MAIN_LOOP_STATE_IO_PULL;
+  item->callback = pull_callback;
+  item->user_data = user_data;
+  item->is_alive = TRUE;
+
+  switch (ring) {
+    case (S_MAIN_LOOP_RING_ONE):
+      s_linked_list_append (self->ring_1_io_pull_list, item);
+      break;
+    case (S_MAIN_LOOP_RING_TWO):
+      s_linked_list_append (self->ring_2_io_pull_list, item);
+      break;
+    default:
+      s_err_print ("Invalid value of the ring parameter for "
+                   "s_main_loop_add_ function.");
+  }
+  s_mutex_unlock (self->mutex[ring][S_MAIN_LOOP_STATE_IO_PULL]);
+}
+
+void
+s_main_loop_add_worker (SMainLoop * self,
+                        SMainLoopRing ring,
+                        RunFunc worker_callback,
+                        spointer user_data) {
+  //
+  s_mutex_lock (self->mutex[ring][S_MAIN_LOOP_STATE_DO_WORKERS]);
+  s_dbg_print ("locked");
+  
+  SMainLoopItem * item = malloc (sizeof (SMainLoopItem));
+  item->state = S_MAIN_LOOP_STATE_DO_WORKERS;
+  item->callback = worker_callback;
+  item->user_data = user_data;
+  item->is_alive = TRUE;
+
+  switch (ring) {
+    case (S_MAIN_LOOP_RING_ONE):
+      s_linked_list_append (self->ring_1_workers_list, item);
+      break;
+    case (S_MAIN_LOOP_RING_TWO):
+      s_linked_list_append (self->ring_2_workers_list, item);
+      break;
+    default:
+      s_err_print ("Invalid value of the ring parameter for "
+                   "s_main_loop_add_worker function.");
+  }
+  s_mutex_unlock (self->mutex[ring][S_MAIN_LOOP_STATE_DO_WORKERS]);
+  s_dbg_print ("Unlocked");
+}
+
+void
+s_main_loop_add_teardown_handler (SMainLoop * self,
+                                  Callback teardown_handler,
+				                          spointer self_obj,
+				                          spointer user_data) {
+  s_mutex_lock (self->teardown_handler_mutex);
+  if (self == NULL) {
+    s_err_print ("No self pointer provided for the teardown handler "
+                 "This is an error.");
+    print_backtrace ();
+  }
+
+  SMainLoopTeardownHandler * item = malloc (sizeof (SMainLoopTeardownHandler));
+
+  item->teardown_handler = teardown_handler;
+  item->obj_self = self_obj;
+  item->user_data = user_data;
+
+  s_linked_list_append (self->teardown_handler_list, item);
+  s_mutex_unlock (self->teardown_handler_mutex);
+}
+
+/* ************************************************************************** */
+
+void
+_s_main_loop_teardown_handerer_obj_free (SMainLoopTeardownHandler * self) {
+  Callback callback = self->teardown_handler;
+  callback (self->obj_self, self->user_data);
+  free (self);
+}
+
+void
+_s_main_loop_do_list (SLinkedList * event_handler_list) {
+  if (s_linked_list_len (event_handler_list) == 0) {
+    return;
+  }
+
+  s_linked_list_head (event_handler_list);
+  do {
+    SMainLoopItem * item = s_linked_list_get_current (event_handler_list);
+    RunFunc callback = item->callback;
+    spointer user_data = item->user_data;
+
+    item->is_alive  = callback (user_data);
+  } while (s_linked_list_next (event_handler_list));
+
+  s_linked_list_head (event_handler_list);
+  do {
+    SMainLoopItem * item = s_linked_list_get_current (event_handler_list);
+    if (!item->is_alive) {
+      s_linked_list_remove_current (event_handler_list, TRUE);
+    }
+  } while (s_linked_list_next (event_handler_list));
+
+}
+
+void
+_s_main_loop_do_ring_two (SMainLoop * self) {
+  switch (self->ring_2_state) {
+    case (S_MAIN_LOOP_STATE_EVENT):
+      _s_main_loop_do_list (self->ring_2_event_handler_list);
+      self->ring_2_state = S_MAIN_LOOP_STATE_IO_PUSH;
+      break;
+    case (S_MAIN_LOOP_STATE_IO_PUSH):
+      _s_main_loop_do_list (self->ring_2_io_push_list);
+      self->ring_2_state = S_MAIN_LOOP_STATE_IO_PULL;
+      break;
+    case (S_MAIN_LOOP_STATE_IO_PULL):
+      _s_main_loop_do_list (self->ring_2_io_pull_list);
+      self->ring_2_state = S_MAIN_LOOP_STATE_DO_WORKERS;
+      break;
+    case (S_MAIN_LOOP_STATE_DO_WORKERS):
+      _s_main_loop_do_list (self->ring_2_workers_list);
+      self->ring_2_state = S_MAIN_LOOP_STATE_EVENT;
+      break;
+    default:
+      s_err_print ("Reaching a place that should not be reached.\n"
+                   "Aborting.");
+      exit (666);
+  }
+}
+
+

=== modified file 'libssts/MainLoop.h'
--- libssts/MainLoop.h	2013-09-04 20:52:28 +0000
+++ libssts/MainLoop.h	2016-01-28 20:12:20 +0000
@@ -0,0 +1,318 @@
+#ifndef __H_MAIN_LOOP__
+#define __H_MAIN_LOOP__
+
+/*
+Copyright (c) 2013-2016 Gustav Hartvigsson
+
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+ */
+
+#include "defs.h"
+#include "Func.h"
+#include "utils.h"
+#include "utils.h"
+
+S_BEGIN_DECLS
+
+/**
+ * @file
+ * @defgroup SMainLoop SMainLoop
+ * @addtogroup SMainLoop
+ * @{
+ * The main loop system consits of two rings. The first ring is for high
+ * priority stuffs that need to be run often. The second ring is run much less
+ * often than the first ring.
+ *
+ * The sectond ring is put in the next state only after the first ring has done
+ * what is required of it.
+ *
+ * Please note that if you are doing pull/push of data via streams, only use one
+ * or the other, do not mix ring 1 and ring 2 as that may lead to filling
+ * or depleating of buffers.
+ *
+ * Here is an overwiev of the main loop:
+ * @code
+ *  Ring 1
+ *   [Do event handlers] -> [IO Push] -> [IO Pull] -> [Do Workers] ->
+ *       ^                                                  |
+ *       |                                                  |
+ *       -------------- [Sleep] <---  [Ring 2 Next] <--------
+ *
+ *  Ring 2
+ *   [Do event handlers] -> [IO Push] -> [IO PULL]
+ *         ^                                |
+ *         |                                |
+ *         ---------  [Do Workers] <---------
+ * @endcode
+ */
+
+/* -----------------------------------------------------------------------------
+ * declarations
+ * -----------------------------------------------------------------------------
+ */
+
+/**
+ * The structure that represents the main loop. It is seldom required to
+ * directly interact with objcets of this type.
+ */
+typedef struct SMainLoop SMainLoop;
+
+/**
+ * The stats that the main loop can be in.
+ */
+typedef enum {
+  S_MAIN_LOOP_STATE_NONE,
+  S_MAIN_LOOP_STATE_EVENT,
+  S_MAIN_LOOP_STATE_IO_PUSH,
+  S_MAIN_LOOP_STATE_IO_PULL,
+  S_MAIN_LOOP_STATE_DO_WORKERS,
+  S_MAIN_LOOP_STATE_RING_TWO_NEXT,
+  S_MAIN_LOOP_STATE_SLEEP,
+  S_MAIN_LOOP_STATE_LAST
+} SMainLoopState;
+
+/**
+ * The namse of the states.
+ */
+S_UNUSED static schar *
+SMainLoopStateName [] = {
+  "NONE",
+  "Event",
+  "Push",
+  "Pull",
+  "Do Work",
+  "Ring Two Next",
+  "Sleep",
+  "NONE",
+  0x0,
+  0x0
+};
+
+/**
+ * Get the name of the state. For use in bindings.
+ */
+schar *
+s_main_loop_state_get_name (SMainLoopState state);
+
+/**
+ * The different rings.
+ */
+typedef enum {
+  S_MAIN_LOOP_RING_NONE,
+  S_MAIN_LOOP_RING_ONE,
+  S_MAIN_LOOP_RING_TWO,
+  S_MAIN_LOOP_RING_LAST
+} SMainLoopRing;
+
+/**
+ * The names of the rings
+ */
+S_UNUSED static schar *
+SMainLoopRingName [] = {
+  "NONE",
+  "Ring One",
+  "Ring Two",
+  "NONE",
+  0x0,
+  0x0
+};
+
+/**
+ * Get name of the ring. For used in bindings.
+ */
+schar *
+s_main_loop_ring_get_name (SMainLoopRing ring);
+
+/* -----------------------------------------------------------------------------
+ * definition of the real functions and stuffs.
+ * -----------------------------------------------------------------------------
+ */
+
+/** @brief
+ * Create a new MainLoop.
+ *
+ * This is something that usually is not needed, and unless you know exectly
+ * what you are doing, it is never recomeded to create your own mainloops,
+ * instead use @ref s_main_loop_get_default.
+ */
+SMainLoop *
+s_main_loop_new ();
+
+/** @brief
+ * Get the default mainloop.
+ *
+ * The default mainloop is the mainloop that it is recomeded to interact with.
+ */
+SMainLoop *
+s_main_loop_get_default ();
+
+/** @brief
+ * Run a mainloop.
+ *
+ * This is usually the way that applications are started, after all pre-start
+ * stuffs are done.
+ *
+ * @param loop The mainloop to be run.
+ */
+void
+s_main_loop_run (SMainLoop * loop);
+
+
+/** @brief
+ * Quit the mainloop. This is usually the way that applications are quit.
+ *
+ * When this function is run, it is important that all data has been freed.
+ * To assure that this has happened you can add teardown handlers to the
+ * main loop using the @ref s_main_loop_add_teardown_handler function,
+ * these handlers will be run when s_main_loop_quit is run.
+ */
+void
+s_main_loop_quit (SMainLoop * loop);
+
+/** @brief
+ * Add a reference to the main loop.
+ *
+ * This may or may not be useful. When the reference count reaches zero the
+ * main loop will be free using @ref s_main_loop_quit.
+ */
+void
+s_main_loop_ref (SMainLoop * loop);
+
+/** @brief
+ * Remove a reference to the main loop.
+ *
+ * This may or may not be useful. When the reference count reaches zero the
+ * main loop will be free using @ref s_main_loop_quit.
+ */
+void
+s_main_loop_unref (SMainLoop * loop);
+
+/** @brief.
+ * Add an event handler to the main loop
+ *
+ * @param loop The main loop to add the event hadler to.
+ * @param ring To what ring to add the event handler to. (Ring 1 for high
+ *             priority handlers, and ring 2 for low priority handlers.)
+ * @param event_handler The event handler to be added to the main loop.
+ * @param user_data The user data to be passed to the event handler. This
+ *                  could be data that is relevent to the componant where the
+ *                  event handler originates from.
+ */
+void
+s_main_loop_add_event_handler (SMainLoop * loop,
+			       SMainLoopRing ring,
+			       RunFunc event_handler,
+			       spointer user_data);
+
+/** @brief
+ * Add an I/O push event to the main loop
+ *
+ * @param loop the main loop to add the I/O push event to.
+ * @param ring The ring to add the I/O push event to.
+ * @param push_callback The callback that prepforms the I/O push.
+ * @param user_data A pointer to the data to be provided to the callback.
+ */
+void
+s_main_loop_add_io_push (SMainLoop * loop,
+			 SMainLoopRing ring,
+			 RunFunc push_callback,
+			 spointer user_data);
+
+/** @brief
+ * Add a I/O pull event to the main loop.
+ *
+ * @param loop The loop to add the I/O pull event to.
+ * @param ring The ring to add the evet to.
+ * @param pull_callback The callback to be performs the I/O pull.
+ * @param user_data The data to be provided to the callback.
+ */
+void
+s_main_loop_add_io_pull (SMainLoop * loop,
+			 SMainLoopRing ring,
+			 RunFunc pull_callback,
+			 spointer user_data);
+
+/** @brief
+ * Add a worker to the main loop.
+ *
+ * @param loop The loop to add the worker to.
+ * @param ring The ring to add the worker to.
+ * @param pull_callback The worker callback.
+ * @param user_data The data to be provided to the callback.
+ */
+void
+s_main_loop_add_worker (SMainLoop * loop,
+			SMainLoopRing ring,
+			RunFunc worker_callback,
+			spointer user_data);
+
+/** @brief
+ * Add a teardown handler.
+ *
+ * A teardown handler is a function that
+ *
+ * @param loop The loop to add the teardown handler to.
+ * @param teardown_handler The teardown handler.
+ * @param self The self object that is to be torn down.
+ * @param user_data A pointer to be provided to the callback.
+ */
+void
+s_main_loop_add_teardown_handler (SMainLoop * loop,
+				  Callback teardown_handler,
+				  spointer self,
+				  spointer user_data);
+
+
+/**
+ * @section THREAD_SAFTEY THREAD SAFTEY
+ */
+inline void FOOL_DOXYGEN ();
+
+/** @brief
+ * lock the main loop form tampering from other threads.
+ *
+ * This function locks the main loop's mutex so that no changes can
+ * be done to the loop whilst it is locked.
+ *
+ * @param loop The main loop to be locked.
+ */
+void
+s_main_loop_lock (SMainLoop * loop);
+
+
+/** @brief
+ * lock the main loop form tampering from other threads.
+ *
+ * This function locks the main loop's mutex so that no changes can
+ * be done to the loop whilst it is locked.
+ *
+ * @param loop The loop to be unlocked.
+ */
+void
+s_main_loop_unlock (SMainLoop * loop);
+
+
+/**
+ * @}
+ */
+
+S_END_DECLS
+
+
+#endif /* __H_MAIN_LOOP__ */

=== modified file 'libssts/Map.c'
--- libssts/Map.c	2016-01-26 12:42:50 +0000
+++ libssts/Map.c	2016-01-28 20:12:20 +0000
@@ -97,13 +97,13 @@
    */
   self->free_key = free_key;
   self->free_value = free_value;
-  
+
   /*
    * Creating a place to store the items.
    */
   self->array = s_dynamic_array_new (S_MAP_DEFAULT_NUMBER_OF_BUCKETS,
                                      FREEFUNC(_internal_s_map_free_buckets));
-  
+
   /* if no func is set we have to use some other metod of
    */
   if (key_hash_func == NULL){
@@ -117,11 +117,11 @@
 
 void
 s_map_free (SMap * self, sboolean free_data) {
-  
+
   s_map_for_each (self,
                   FOREACHFUNC(_s_map_internal_free_map_items_for_each),
                   free_data);
-  
+
   s_dynamic_array_free (self->array, TRUE);
   free (self);
 }

=== modified file 'libssts/Thread.c'
--- libssts/Thread.c	2015-10-28 14:45:22 +0000
+++ libssts/Thread.c	2016-01-29 17:49:29 +0000
@@ -20,13 +20,17 @@
 #endif
 }
 
+char *
+s_thread_status_get_name (SThreadStatus status) {
+  return SThreadStatusName[status];
+}
 
 /* ****************************************************************************
  ********************************** SMutex ************************************
  **************************************************************************** */
 
 struct SMutex {
-  mtx_t mutex;
+  mtx_t *  mutex;
   _Atomic(sboolean) locked;
 };
 
@@ -34,40 +38,51 @@
 SMutex *
 s_mutex_new () {
   SMutex * self = malloc (sizeof (SMutex));
-
+  
+  self->mutex = malloc (sizeof (mtx_t));
+  
   atomic_init(&(self->locked), FALSE);
-
-  mtx_init (&(self->mutex), mtx_plain);
-
-  return self;
+  
+  sint status = mtx_init (self->mutex, mtx_plain);
+  
+  if (status == thrd_success) {
+    return self;
+  }
+  free (self);
+  s_err_print ("Could not create thrad. Error: %s.\n",
+               s_thread_status_get_name (status));
+  print_backtrace ();
+  return NULL;
 }
 
 void
 s_mutex_free (SMutex * self) {
-  mtx_destroy (&(self->mutex));
+  mtx_destroy (self->mutex);
+  free (self->mutex);
   free (self);
 }
 
-sint
+SThreadStatus
 s_mutex_lock (SMutex * self) {
-  sint ret_val = mtx_lock (&(self->mutex));
+  sint ret_val = mtx_lock (self->mutex);
   atomic_store(&(self->locked), TRUE);
   return ret_val;
 }
 
-sint
+SThreadStatus
 s_mutex_unlock (SMutex * self) {
-  sint ret_val = mtx_unlock (&(self->mutex));
+  sint ret_val = mtx_unlock (self->mutex);
   atomic_store(&(self->locked), FALSE);
   return ret_val;
 }
 
-sboolean
+SThreadStatus
 s_mutex_check_lock (SMutex * self) {
   return atomic_load(&(self->locked));
 }
 
 
+
 /* ****************************************************************************
  ********************************** SThread ***********************************
  **************************************************************************** */
@@ -86,7 +101,7 @@
 SThread *
 s_thread_new (RunFunc func) {
   SThread * self = malloc (sizeof (SThread));
-
+  
   self->func = func;
 
   atomic_store (&(self->is_running), FALSE);

=== modified file 'libssts/Thread.h'
--- libssts/Thread.h	2015-11-14 17:14:26 +0000
+++ libssts/Thread.h	2016-01-29 17:49:29 +0000
@@ -40,17 +40,59 @@
 void
 s_usleep (slong us);
 
+/** @brief
+ * The status of the thread.
+ *
+ * These values are equivalent to the value defined my the threads.h. They are
+ * offten used as return values for different fuctions.
+ */
+typedef enum {
+  S_THREAD_STATUS_SUCCESS    = thrd_success, /*< Succes*/
+  S_THREAD_STATUS_TIMEOUT    = thrd_timeout, /*< Timeout*/
+  S_THREAD_STATUS_ERROR      = thrd_error,   /*< Error */
+  S_THREAD_STATUS_BUSY       = thrd_busy,    /*< Busy */
+  S_THREAD_STATUS_NO_MEM     = thrd_nomem,   /*< No memory*/
+  S_THREAD_STATUS_LAST                       /*< Not used */
+} SThreadStatus;
+
+/**
+ * @see SThreadStatus
+ */
+S_UNUSED static
+schar * SThreadStatusName[] = {
+  "Succes",
+  "Timeout",
+  "Error",
+  "Busy",
+  "No Momory",
+  "INVALID",
+  0x0,
+  0x0
+};
+
+/** @brief
+ * Get the status name.
+ *
+ * For use in bindings.
+ *
+ * @see SThreadStatus
+ * @see SThreadStatusName
+ */
+
+char *
+s_thread_status_get_name (SThreadStatus status);
 
 /* ****************************************************************************
  ********************************** SMutex ************************************
  **************************************************************************** */
 
 /**
- * @defgroup Mutex Mutex
- * @addtogroup Mutex
+ * @defgroup SMutex SMutex
+ * @addtogroup SMutex
  * @{
  */
 
+
 /**
  * An SMutex is an opaque data type that handles the platform specifics of the
  * Mutex, if it exists. If not we roll our own.
@@ -74,7 +116,7 @@
  *
  * Returns a key used to unlock the mutex.
  */
-sint
+SThreadStatus
 s_mutex_lock (SMutex * self);
 
 /**
@@ -83,7 +125,7 @@
  * @param self The mutex to unlock;
  * @param key The key used to unlock the mutex.
  */
-sint
+SThreadStatus
 s_mutex_unlock (SMutex * self);
 
 /**
@@ -91,7 +133,7 @@
  *
  * Note that this operation is non-atomic and may be wrong. 
  */
-sboolean
+SThreadStatus
 s_mutex_check_lock (SMutex * self);
 
 /** @} */

=== modified file 'libssts/baseobject.c'
--- libssts/baseobject.c	2015-10-28 14:45:22 +0000
+++ libssts/baseobject.c	2016-01-28 20:12:20 +0000
@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2013-2014 Gustav Hartvigsson
+Copyright (c) 2013-2016 Gustav Hartvigsson
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal

=== modified file 'libssts/defs.h'
--- libssts/defs.h	2015-11-14 17:14:26 +0000
+++ libssts/defs.h	2016-01-28 20:12:20 +0000
@@ -118,7 +118,7 @@
 
 /**
  * uchar is the representation for use with wide unicode strings and what not.
- * 
+ *
  * The rationell for using char32_t as the unicode wide char instead of
  * wchar_t is that wchar_t's size depends on the imprementation, compiler,
  * and system.
@@ -127,7 +127,7 @@
  * other communication we set this as the standard for the uchar.
  *
  * @warning This is not an unsigned char. If you need to store such small
- * values use #subyte 
+ * values use #subyte
  */
 typedef char32_t suchar;
 
@@ -226,7 +226,7 @@
 /** @brief
  * The names of the SType's
  *
- * 
+ *
  */
 static char * STypeName[] S_UNUSED = {
   "NONE",

=== modified file 'tests/CMakeLists.txt'
--- tests/CMakeLists.txt	2016-01-26 12:26:20 +0000
+++ tests/CMakeLists.txt	2016-01-28 20:12:20 +0000
@@ -15,6 +15,8 @@
   map.c
   callback.h
   callback.c
+  mainloop.h
+  mainloop.c
   thread.h
   thread.c
 )

=== modified file 'tests/callback.c'
--- tests/callback.c	2016-01-26 14:24:15 +0000
+++ tests/callback.c	2016-01-28 20:12:20 +0000
@@ -39,21 +39,22 @@
                           S_CALLBACK_CALLBACK);
 
   s_object_install_callbacks (obj, TEST_CALLBACK_END, callback_entries);
-  
+
   test_case (s_object_notify (obj, "test1", NULL) == 2,
             "the output from the callback is the expected value");
-  
+
   sint * output = malloc (sizeof (sint));
-  
+
   s_object_notify (obj, "test2", (spointer)output);
-  
+
   test_case (*output != NULL, "The value that was givef vida the user_data"
                               " pointer is not null");
-  
+
   test_case (*output == 1337, "The value that was given via the"
                               " user_data pointer has the corret"
                               " value");
-  
+  free (output);
+
   s_object_free (obj);
 
   end_unit ();

=== modified file 'tests/main.c'
--- tests/main.c	2016-01-26 12:42:50 +0000
+++ tests/main.c	2016-01-28 20:12:20 +0000
@@ -13,10 +13,11 @@
 #include "linked_list.h"
 #include "thread.h"
 #include "map.h"
+#include "mainloop.h"
 
 int main  (int argc, char ** argv, char ** argp) {
   setup_suite ("SSTS");
-  
+
   test_unit (test_refcount, "Refcount");
   test_unit (test_to_string, "To String");
   test_unit (test_error, "Error");
@@ -24,6 +25,7 @@
   test_unit (test_linked_list, "Linked List");
   test_unit (test_map, "Map");
   test_unit (test_callback, "Callback");
+  test_unit (test_main_loop, "Main Loop");
   //test_unit (test_thread, "Threading");
 
   end_suite ();

=== added file 'tests/mainloop.c'
--- tests/mainloop.c	1970-01-01 00:00:00 +0000
+++ tests/mainloop.c	2016-01-29 17:49:29 +0000
@@ -0,0 +1,58 @@
+#include "mainloop.h"
+#include <SimpleTypeSystem.h>
+
+sboolean
+_do_some_work_1 (sint * itt) {
+  s_print ("Hello from Worker 1! itt:%d\n", *itt);
+  *itt += 1;
+  if (*itt >= 100) {
+    return FALSE;
+  } else {
+    return TRUE;
+  }
+}
+
+sboolean
+_do_some_work_2 (sint * itt) {
+  s_print ("Hello from Worker 2! itt:%d\n", *itt);
+  *itt += 1;
+  if (*itt >= 100) {
+    return FALSE;
+  } else {
+    return TRUE;
+  }
+}
+
+void
+_do_teardown (sint * itt, spointer user_data) {
+  free (itt);
+}
+
+int test_main_loop (void) {
+  setup_unit ();
+
+  SMainLoop * loop = NULL;
+  loop = s_main_loop_get_default ();
+  
+  test_case (loop != NULL, "The loop is not NULL");
+  
+  sint * itt = malloc (sizeof (sint));
+  *itt = 0;
+  
+  s_main_loop_add_teardown_handler (loop, CALLBACK (_do_teardown), itt, NULL);
+  
+  s_main_loop_add_worker (loop, S_MAIN_LOOP_RING_ONE, RUNFUNC(_do_some_work_1), itt);
+
+
+  s_main_loop_add_worker (loop, S_MAIN_LOOP_RING_TWO, RUNFUNC(_do_some_work_2), itt);
+
+  s_main_loop_run (loop);
+  
+  s_print ("Value: %d\n", *itt);
+  
+  test_case (*itt >= 100, "The avlue of the itterator is ");
+
+  s_main_loop_quit (loop);
+  
+  end_unit ();
+}

=== added file 'tests/mainloop.h'
--- tests/mainloop.h	1970-01-01 00:00:00 +0000
+++ tests/mainloop.h	2016-01-28 20:12:20 +0000
@@ -0,0 +1,5 @@
+#ifndef __H_TEST_MAIN_LOOP__
+#define __H_TEST_MAIN_LOOP__
+#include "test_macros.h"
+int test_main_loop (void);
+#endif

